/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// we use #import to make sure the header is only read in once

#import "SyncController.h"
#import "customization.h"
#import "constants.h"
#import "ClientConfig.h"
#include "constants.h" // costants and defines for Funambol sync clien and UI components
#include "utils.h"
#include "SyncQueueManager.h"

@implementation SyncController

- (id)init {
    self = [super init];    
    return self;
}

- (void) dealloc 
{
    [scheduledSyncManager dealloc];
    [super dealloc];
}

- (void) awakeFromNib
{
    if (!(ClientConfig::getInstance()->getIsInitialized())) {
        ClientConfig::getInstance()->init();
        ClientConfig::getInstance()->read();
    }

    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    [syncWithServer setStringValue:NSLocalizedStringFromTableInBundle(@"syncWithServerText",nil,b, @"-")];
    [syncDirection setStringValue:NSLocalizedStringFromTableInBundle(@"syncDirectionText",nil,b, @"-")];
    [showInMenuBar setTitle:NSLocalizedStringFromTableInBundle(@"showStatusInMenuBarText",nil,b, @"-")];

    
    if (SyncQueueManager::isSyncRunning()) {
        [syncButton setTitle:NSLocalizedStringFromTableInBundle(@"cancelSyncMenuItem",nil,b, @"-")];
    } else { 
        [syncButton setTitle:NSLocalizedStringFromTableInBundle(@"syncNowText",nil,b, @"-")];
    }
    
    if (ClientConfig::getInstance()->readMenubarUIEnabled()) {
        menuBarUiEnabled = YES;
        [showInMenuBar setState:NSOnState];
    } else {
        menuBarUiEnabled = NO;
        [showInMenuBar setState:NSOffState];
    }
    
  //  NSArray* schedulevalues = [[NSArray alloc] init];
    NSString* scheduleNever = NSLocalizedStringFromTableInBundle(@"never",nil,b, @"-");
    NSString* schedule1 = NSLocalizedStringFromTableInBundle(@"15min",nil,b, @"-");
    NSString* schedule2 = NSLocalizedStringFromTableInBundle(@"30min",nil,b, @"-");
    NSString* schedule3 = NSLocalizedStringFromTableInBundle(@"1h",nil,b, @"-");
    NSString* schedule4 = NSLocalizedStringFromTableInBundle(@"2h",nil,b, @"-");
    NSString* schedule5 = NSLocalizedStringFromTableInBundle(@"4h",nil,b, @"-");
    NSString* schedule6 = NSLocalizedStringFromTableInBundle(@"6h",nil,b, @"-");
    NSString* schedule7 = NSLocalizedStringFromTableInBundle(@"8h",nil,b, @"-");
    NSString* schedule8 = NSLocalizedStringFromTableInBundle(@"12h",nil,b, @"-");
    NSString* schedule9 = NSLocalizedStringFromTableInBundle(@"1d",nil,b, @"-");
    
    [scheduleBox addItemWithTitle:scheduleNever];
    [scheduleBox addItemWithTitle:schedule1];
    [scheduleBox addItemWithTitle:schedule2];
    [scheduleBox addItemWithTitle:schedule3];
    [scheduleBox addItemWithTitle:schedule4];
    [scheduleBox addItemWithTitle:schedule5];
    [scheduleBox addItemWithTitle:schedule6];
    [scheduleBox addItemWithTitle:schedule7];
    [scheduleBox addItemWithTitle:schedule8];
    [scheduleBox addItemWithTitle:schedule9];
    
    ClientConfig::getInstance()->read();
    
    int schedulingValue = ClientConfig::getInstance()->getSchedulingInterval(CONTACTS_SOURCE_NAME);
    
    switch (schedulingValue) {
        case 0://never
            [scheduleBox selectItemAtIndex:0];
            break;
        case 15://15min
            [scheduleBox selectItemAtIndex:1];
            break;
        case 30://30min
            [scheduleBox selectItemAtIndex:2];
            break;
        case 60://1h
            [scheduleBox selectItemAtIndex:3];
            break;
        case 120://2h
            [scheduleBox selectItemAtIndex:4];
            break;
        case 240://4h
            [scheduleBox selectItemAtIndex:5];
            break;
        case 360://6h
            [scheduleBox selectItemAtIndex:6];
            break;
        case 480://8h
            [scheduleBox selectItemAtIndex:7];
            break;
        case 720://12h
            [scheduleBox selectItemAtIndex:8];
            break;
        case 1440:
            [scheduleBox selectItemAtIndex:9];
            break;
        default:
            break;
    }
   // [scheduleBox setObjectValue:[scheduleBox objectValueOfSelectedItem]];
    
    [scheduleNever release];
    [schedule1 release];
    [schedule2 release];
    [schedule3 release];
    [schedule4 release];
    [schedule5 release];
    [schedule6 release];
    [schedule7 release];
    [schedule8 release];
    [schedule9 release];
    
    NSString* uploadOnly = NSLocalizedStringFromTableInBundle(@"uploadOnly",nil,b, @"-");
    NSString* downloadOnly = NSLocalizedStringFromTableInBundle(@"downloadOnly",nil,b, @"-");
    NSString* twoway = NSLocalizedStringFromTableInBundle(@"twoway",nil,b, @"-");

    
    [direction addItemWithTitle:twoway];
    [direction addItemWithTitle:uploadOnly];
    [direction addItemWithTitle:downloadOnly];
    
    [uploadOnly release];
    [downloadOnly release];
    [twoway release];
    
    SyncSourceConfig* ssc = ClientConfig::getInstance()->getSyncSourceConfig(CONTACTS_SOURCE_NAME);
    
     const char* syncDirectionValue = ssc->getSync();
    if ( strcmp(syncDirectionValue, "one-way-from-client") == 0 ){
        [direction selectItemAtIndex:1]; //first item is at value 0
    }else if ( strcmp(syncDirectionValue, "one-way-from-server") == 0){
        [direction selectItemAtIndex:2];
    }else{ //two way
        [direction selectItemAtIndex:0];
    }

    // set up launchctl configuration manager for scheduled sync
    scheduledSyncManager = [[LaunchCtlConfigManager alloc] init];
    
    // get distribuited NSCenter singleton
    nCenter = [NSDistributedNotificationCenter defaultCenter];
    
    [nCenter addObserver:self  
                selector:@selector(syncStartedCallBack:)
                    name:@CLIENT_SYNC_START_NOTIFICATION
                  object:nil];

    [nCenter addObserver:self  
                selector:@selector(syncCancelledCallBack:)
                    name:@SYNC_CANCELLED_NOTIFICATION
                  object:nil];    
    
    [nCenter addObserver:self  
                selector:@selector(syncEndedCallBack:)
                    name:@CLIENT_SYNC_ENDED_NOTIFICATION
                  object:nil];
}

- (NSString*) getApplicationName {
    
    NSString* appName = [NSString stringWithUTF8String:APPNAME];
    return appName;
}

- (IBAction)syncWithServerAction:(id)pId
{
    
    int val = [pId indexOfSelectedItem];
    StringBuffer configValStr;
    int currentConfigVal = 0, configVal = 0;
    
   currentConfigVal = ClientConfig::getInstance()->getSchedulingInterval(CONTACTS_SOURCE_NAME);
    
    switch (val) {
        case 0://never
            configVal = 0;
            break;
        case 1://15min
            configVal = 15;
            break;
        case 2://30min
            configVal = 30;
            break;
        case 3://1h
            configVal = 60;
            break;
        case 4://2h
            configVal = 120;
            break;
        case 5://4h
            configVal = 240;
            break;
        case 6://6h
            configVal = 360;
            break;
        case 7://8h
            configVal = 480;
            break;
        case 8://12h
            configVal = 720;
            break;
        case 9:
            configVal = 1440;
            break;
        default:
            break;
    }
    
    if (currentConfigVal != configVal) {
        configValStr.sprintf("%d",configVal);
        ClientConfig::getInstance()->setSchedulingInterval(CONTACTS_SOURCE_NAME, configValStr.c_str());
        
        if (configVal) { // set new scheduled sync (converting value in seconds)
            [scheduledSyncManager setNewScheduledSync: (configVal * 60)];
        } else { // remove scheduled sync
            [scheduledSyncManager removeScheduledSync];
        }
    }
}

- (IBAction)syncDirectionAction:(id)pId
{
    int val = [pId indexOfSelectedItem];
    StringBuffer configVal;
 
    switch (val) {
        case 0:
            configVal = "two-way";
            break;
        case 1:
            configVal = "one-way-from-client";
            break;
        case 2:
            configVal = "one-way-from-server";
            break;
        default:
            break;
    }

    ClientConfig::getInstance()->getSyncSourceConfig(CONTACTS_SOURCE_NAME)->setSync(configVal.c_str());
    ClientConfig::getInstance()->safeSave();
}

- (IBAction)showStatusInMenuBarAction:(id) pId
{
    if (menuBarUiEnabled) {
        menuBarUiEnabled = NO;
        ClientConfig::getInstance()->saveMenubarUIEnabled(false);
        [self hideMenubarUi];
    } else {
        menuBarUiEnabled = YES;
        ClientConfig::getInstance()->saveMenubarUIEnabled(true);
        [self showMenuBarUi];
    }    
}

-(void)checkSyncProcessStatus:(NSNotification *) aNotification 
{
    NSBundle* b = [NSBundle bundleForClass:[self class]];

    [synctask release];
    [[NSNotificationCenter defaultCenter]  removeObserver: self];
    [syncButton setTitle:NSLocalizedStringFromTableInBundle(@"syncNowText",nil,b, @"-")];
}

-(void)syncStartedCallBack:(NSNotification *) aNotification 
{
    NSBundle* b = nil;
    
    [scheduleBox setEnabled: NO];
    [direction setEnabled: NO];
    
    b = [NSBundle bundleForClass:[self class]];
    [syncButton setTitle:NSLocalizedStringFromTableInBundle(@"cancelSyncMenuItem",nil,b, @"-")];
}

-(void) syncCancelledCallBack:(NSNotification *) aNotification
{
    [syncButton setEnabled: NO];
}

-(void)syncEndedCallBack:(NSNotification *) aNotification 
{
    NSBundle* b = nil;
    
    [scheduleBox setEnabled: YES];
    [direction setEnabled: YES];
    
    b = [NSBundle bundleForClass:[self class]];
    
    [syncButton setTitle:NSLocalizedStringFromTableInBundle(@"syncNowText",nil,b, @"-")];
    [syncButton setEnabled: YES];
}

- (void) startSync
{
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    [syncButton setTitle:NSLocalizedStringFromTableInBundle(@"cancelSyncMenuItem",nil,b, @"-")];
    synctask = [[NSTask alloc]init];
    
    // register app as an observer for sync client
    [[NSNotificationCenter defaultCenter] addObserver:self  
                                             selector:@selector(checkSyncProcessStatus:)
                                                 name:NSTaskDidTerminateNotification
                                               object:synctask];
    
    
    NSString* arg = @"-a";
    NSMutableArray* args = [NSMutableArray array];
    [args addObject:arg];
    NSString* path = [[NSString alloc] initWithFormat:@"%s", SYNC_APPLICATION_PATH];
    [synctask setLaunchPath:path];
    [synctask setArguments:args];
    [synctask launch];
}

- (IBAction)firstSyncOkAction:(id)pId{
    [self startSync];
    [NSApp endSheet:firstSyncWarning ];
}

- (IBAction)firstSyncCancelAction:(id)pId{
    [NSApp endSheet:firstSyncWarning ];
}

- (IBAction)emptyOkAction:(id)pId{
    [NSApp endSheet:emptyWarning ];
    [tabViewTemp selectTabViewItem:accountTab];
}

- (void)modalSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo{
    [sheet orderOut:self];
}


- (IBAction)syncAddressBookAction:(id)pId{
    NSBundle* b = [NSBundle bundleForClass:[self class]];

    if(ClientConfig::getInstance()->areCredentialSaved()){
    
        if([[syncButton title]isEqual:NSLocalizedStringFromTableInBundle(@"syncNowText",nil,b, @"-")]){
            if(ClientConfig::getInstance()->areCredentialValid() == false) {
                //NSString* text = [NSLocalizedStringFromTableInBundle(@"firstSyncMessage1",nil,b, @"-") stringByAppendingString:NSLocalizedStringFromTableInBundle(@"firstSyncMessage2",nil,b, @"-")];
                NSString* text =  [NSLocalizedStringFromTableInBundle(@"firstSyncTitle",nil,b, @"-") stringByAppendingString:NSLocalizedStringFromTableInBundle(@"firstSyncMessage1",nil,b, @"-")];
                [firstSyncText setStringValue:text];
                [firstSyncOk setTitle:NSLocalizedStringFromTableInBundle(@"ok",nil,b, @"-")];
                [NSApp beginSheet:emptyCredential modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
                //[tabView selectTabViewItem:accountTab];
            }
        }
        //
        // Warning to be cleared. it should be possible to read the whole config to get the values
        // 
        StringBuffer prop = ClientConfig::getInstance()->readSyncSourceLastTimestamp(CONTACTS_SOURCE_NAME);
    
        if([[syncButton title]isEqual:NSLocalizedStringFromTableInBundle(@"syncNowText",nil,b, @"-")]){
            if(prop.empty() || prop == "0"){
               // NSString* text = [NSLocalizedStringFromTableInBundle(@"firstSyncMessage1",nil,b, @"-") stringByAppendingString:NSLocalizedStringFromTableInBundle(@"firstSyncMessage2",nil,b, @"-")];
                NSString* text =  [NSLocalizedStringFromTableInBundle(@"firstSyncTitle",nil,b, @"-") stringByAppendingString:NSLocalizedStringFromTableInBundle(@"firstSyncMessage1",nil,b, @"-")];
                [firstSyncText setStringValue:text];
                [firstSyncOk setTitle:NSLocalizedStringFromTableInBundle(@"continue",nil,b, @"-")];
                [firstSyncCancel setTitle:NSLocalizedStringFromTableInBundle(@"Cancel",nil,b, @"-")];
                [NSApp beginSheet:firstSyncWarning modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
            }else{
                [self startSync];
            }
        }else{
            // cancel sync: sync button will be disabled by cancelSync notification listner until a sync ended notification is received
            NSMutableDictionary* infoDict = [NSMutableDictionary dictionaryWithCapacity: 1];
            [nCenter postNotificationName: (NSString*)CFSTR(SYNC_CANCELLED_NOTIFICATION) object:nil userInfo: infoDict deliverImmediately: YES];
        }
    }else{
        [emptyText setStringValue:NSLocalizedStringFromTableInBundle(@"EmptyMessage",nil,b, @"-")];
        [emptyOk setTitle:NSLocalizedStringFromTableInBundle(@"ok",nil,b, @"-")];
        [NSApp beginSheet:emptyWarning modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
    }
}

- (void)showMenuBarUi
{
    NSMutableDictionary* infoDict = [NSMutableDictionary dictionaryWithCapacity: 1];
    NSNumber* menuBarStatus = [NSNumber numberWithInt: kMenubarUiGotoActiveStatus];

    [infoDict setObject:(id)menuBarStatus forKey:(id)CFSTR(MENUBAR_STATUS_VALUE)];
    [nCenter postNotificationName: (NSString*)CFSTR(MENUBAR_SET_UI_STATUS_NOTIFICATION) object:nil userInfo: infoDict deliverImmediately: YES];
}

- (void)hideMenubarUi
{
    NSMutableDictionary* infoDict = [NSMutableDictionary dictionaryWithCapacity: 1];
    NSNumber* menuBarStatus = [NSNumber numberWithInt: kMenuBarUiGotoHiddenStatus];

    [infoDict setObject:(id)menuBarStatus forKey:(id)CFSTR(MENUBAR_STATUS_VALUE)];
    [nCenter postNotificationName: (NSString*)CFSTR(MENUBAR_SET_UI_STATUS_NOTIFICATION) object:nil userInfo: infoDict deliverImmediately: YES];
}

@end
